//Winch robot
//
//Code to run on the command computer see:
//WinchBot.c
//
//Source & info: www.HomoFaciens.de


//#include <avr/io.h>
//#include <util/delay.h>
#include <Servo.h>


//Change values below to adapt your motor

#define PULSES_PER_TURN 48

#define STEP_MARGIN_M1 2L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_M1_UP 255    //0 - 255 (125)
#define MIN_DUTYCYCLE_M1_DOWN 140    //0 - 255 (125)
#define MAX_DUTYCYCLE_M1_UP 255    //0 - 255 (255)
#define MAX_DUTYCYCLE_M1_DOWN 180    //0 - 255 (255)

#define STEP_MARGIN_M2 2L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_M2_UP 255    //0 - 255 (125)
#define MIN_DUTYCYCLE_M2_DOWN 140    //0 - 255 (125)
#define MAX_DUTYCYCLE_M2_UP 255    //0 - 255 (255)
#define MAX_DUTYCYCLE_M2_DOWN 180    //0 - 255 (255)

#define STEP_MARGIN_M3 2L       //10 - 1000 (2)
#define MIN_DUTYCYCLE_M3_UP 255    //0 - 255 (125)
#define MIN_DUTYCYCLE_M3_DOWN 140    //0 - 255 (125)
#define MAX_DUTYCYCLE_M3_UP 255    //0 - 255 (255)
#define MAX_DUTYCYCLE_M3_DOWN 180    //0 - 255 (255)


#define M1_S1   A0            //Sensor input 1
#define M1_S2   A1            //Sensor input 2
#define M2_S1   A2            //Sensor input 1
#define M2_S2   A3            //Sensor input 2
#define M3_S1   A4            //Sensor input 1
#define M3_S2   A5            //Sensor input 2

#define M1_DIR   8            //Direction output motor
#define M1_PWM   5            //PWM output motor
#define M2_DIR  12            //Direction output motor
#define M2_PWM  11            //PWM output motor
#define M3_DIR   2            //Direction output motor
#define M3_PWM   6            //PWM output motor

#define SERVO_1  7            //Servo platform
#define SERVO_2  3            //Servo platform
#define SERVO_3 10            //Servo arm rotate
#define SERVO_4  4            //Servo arm tilt
#define SERVO_5  9             //Servo gripper

#define SERVOMIN_1   1
#define SERVOMAX_1 180

#define SERVOMIN_2   1
#define SERVOMAX_2 180

#define SERVOMIN_3   1
#define SERVOMAX_3 180

#define SERVOMIN_4   1
#define SERVOMAX_4 180

#define SERVOMIN_5   1
#define SERVOMAX_5 180

Servo servo_1;
Servo servo_2;
Servo servo_3;
Servo servo_4;
Servo servo_5;

int ServoVal_1;
int ServoVal_2;
int ServoVal_3;
int ServoVal_4;
int ServoVal_5;

int M1SensorRead = 0;
int M1SensorReadOld = 0;
int M1DutyCycle = 100; // 10 - 255
signed long M1SetPoint = 0L;
signed long M1ActualPoint = 0L;
byte M1Step = 0;
byte M1StepDone = 0;

int M2SensorRead = 0;
int M2SensorReadOld = 0;
int M2DutyCycle = 100; // 10 - 255
signed long M2SetPoint = 0L;
signed long M2ActualPoint = 0L;
byte M2Step = 0;
byte M2StepDone = 0;

int M3SensorRead = 0;
int M3SensorReadOld = 0;
int M3DutyCycle = 100; // 10 - 255
signed long M3SetPoint = 0L;
signed long M3ActualPoint = 0L;
byte M3Step = 0;
byte M3StepDone = 0;

byte readByte = 0;
byte readByte2 = 0;

byte verbose = 0;

unsigned long stepTime = 1;

unsigned long millisStart = 0;


void establishContact() {
  while (Serial.available() <= 0) {
    Serial.print('X');   // send a capital X
    delay(300);
  }
}

void printValues() {
  Serial.print("S1 = ");
  Serial.print(ServoVal_1);
  Serial.print(", S2 = ");
  Serial.print(ServoVal_2);
  Serial.print(", S3 = ");
  Serial.print(ServoVal_3);
  Serial.print(", S4 = ");
  Serial.print(ServoVal_4);
  Serial.print(", S5 = ");
  Serial.print(ServoVal_5);
  Serial.print(", M1 = ");
  Serial.print(M1SensorRead);
  Serial.print(", M2 = ");
  Serial.print(M2SensorRead);
  Serial.print(", M3 = ");
  Serial.println(M3SensorRead);
}

void setup(){

  ServoVal_1 = 108;
  ServoVal_2 = 83;
  ServoVal_3 = 76;
  ServoVal_4 = 47;
  ServoVal_5 = 90;

  servo_1.attach(SERVO_1);
  servo_2.attach(SERVO_2);
  servo_3.attach(SERVO_3);
  servo_4.attach(SERVO_4);
  servo_5.attach(SERVO_5);

  pinMode(M1_DIR, OUTPUT);
  pinMode(M1_PWM, OUTPUT);
  analogWrite(M1_PWM, 0);
  digitalWrite(M1_DIR, 0);
  pinMode(M1_S1, INPUT_PULLUP);
  pinMode(M1_S2, INPUT_PULLUP);

  pinMode(M2_DIR, OUTPUT);
  pinMode(M2_PWM, OUTPUT);
  analogWrite(M2_PWM, 0);
  digitalWrite(M2_DIR, 0);
  pinMode(M2_S1, INPUT_PULLUP);
  pinMode(M2_S2, INPUT_PULLUP);

  pinMode(M3_DIR, OUTPUT);
  pinMode(M3_PWM, OUTPUT);
  analogWrite(M3_PWM, 0);
  digitalWrite(M3_DIR, 0);
  pinMode(M3_S1, INPUT_PULLUP);
  pinMode(M3_S2, INPUT_PULLUP);

  stepTime = 0;
  readByte = 0;
  readByte2 = 0;

  M1Step = 0;
  M1StepDone = 1;
  M1SetPoint      = 0L;
  M1ActualPoint   = 0L;
  M1SensorRead    = 0;
  M1SensorReadOld = 0;

  M2Step = 0;
  M2StepDone = 1;
  M2SetPoint      = 0L;
  M2ActualPoint   = 0L;
  M2SensorRead    = 0;
  M2SensorReadOld = 0;

  M3Step = 0;
  M3StepDone = 1;
  M3SetPoint      = 0L;
  M3ActualPoint   = 0L;
  M3SensorRead    = 0;
  M3SensorReadOld = 0;

  verbose = 0;

  // start serial port at 115200 bps:
  Serial.begin(115200);
  establishContact();  // send a byte to establish contact until receiver responds
}


void loop(){

//printValues();// !!!!!!!!!!!!!!!!!!!!!!!!!

  servo_1.write(ServoVal_1);
  servo_2.write(ServoVal_2);
  servo_3.write(ServoVal_3);
  servo_4.write(ServoVal_4);
  servo_5.write(ServoVal_5);


  millisStart = millis();
    do{

      M1SensorReadOld = M1SensorRead;

      M1SensorRead = digitalRead(M1_S1);
      if(digitalRead(M1_S2) == 1){
        M1SensorRead += 2;
      }
      //M1SensorRead = PINC & 0b00000001;
      //if((PINC & 0b00000010) == 2){
      //  M1SensorRead += 2;
      //}

      if(M1SensorRead != M1SensorReadOld){
        if(M1SensorRead == 0 && M1SensorReadOld == 2){
          M1ActualPoint++;
        }
        if(M1SensorRead == 1 && M1SensorReadOld == 0){
          M1ActualPoint++;
        }
        if(M1SensorRead == 3 && M1SensorReadOld == 1){
          M1ActualPoint++;
        }
        if(M1SensorRead == 2 && M1SensorReadOld == 3){
          M1ActualPoint++;
        }

        if(M1SensorRead == 0 && M1SensorReadOld == 1){
          M1ActualPoint--;
        }
        if(M1SensorRead == 1 && M1SensorReadOld == 3){
          M1ActualPoint--;
        }
        if(M1SensorRead == 3 && M1SensorReadOld == 2){
          M1ActualPoint--;
        }
        if(M1SensorRead == 2 && M1SensorReadOld == 0){
          M1ActualPoint--;
        }
      }

      if(abs(M1SetPoint - M1ActualPoint) < STEP_MARGIN_M1){
        analogWrite(M1_PWM, 0);
        digitalWrite(M1_DIR, 0);
        //cbi (PORTB, 0);
        M1StepDone = 1;
        M1DutyCycle = 0;
      }
      else{
        if(M1ActualPoint < M1SetPoint){
          digitalWrite(M1_DIR, 1);
          //sbi (PORTB, 0);
          if(M1DutyCycle < MIN_DUTYCYCLE_M1_UP){
            M1DutyCycle = MIN_DUTYCYCLE_M1_UP;
          }
          analogWrite(M1_PWM, 255 - M1DutyCycle);
        }
        if(M1ActualPoint > M1SetPoint){
          digitalWrite(M1_DIR, 0);
          //cbi (PORTB, 0);
          if(M1DutyCycle < MIN_DUTYCYCLE_M1_DOWN){
            M1DutyCycle = MIN_DUTYCYCLE_M1_DOWN;
          }
          analogWrite(M1_PWM, M1DutyCycle);
        }
      }


      M2SensorReadOld = M2SensorRead;

      M2SensorRead = digitalRead(M2_S1);
      if(digitalRead(M2_S2) == 1){
        M2SensorRead += 2;
      }

      //M2SensorRead = 0;
      //if((PINC & 0b0000100) == 4){
      //  M2SensorRead = 1;
      //}
      //if((PINC & 0b00001000) == 8){
      //  M2SensorRead += 2;
      //}

      if(M2SensorRead != M2SensorReadOld){
        if(M2SensorRead == 0 && M2SensorReadOld == 2){
          M2ActualPoint++;
        }
        if(M2SensorRead == 1 && M2SensorReadOld == 0){
          M2ActualPoint++;
        }
        if(M2SensorRead == 3 && M2SensorReadOld == 1){
          M2ActualPoint++;
        }
        if(M2SensorRead == 2 && M2SensorReadOld == 3){
          M2ActualPoint++;
        }

        if(M2SensorRead == 0 && M2SensorReadOld == 1){
          M2ActualPoint--;
        }
        if(M2SensorRead == 1 && M2SensorReadOld == 3){
          M2ActualPoint--;
        }
        if(M2SensorRead == 3 && M2SensorReadOld == 2){
          M2ActualPoint--;
        }
        if(M2SensorRead == 2 && M2SensorReadOld == 0){
          M2ActualPoint--;
        }
      }

      if(abs(M2SetPoint - M2ActualPoint) < STEP_MARGIN_M2){
        analogWrite(M2_PWM, 0);
        digitalWrite(M2_DIR, 0);
        //cbi (PORTB, 4);
        M2StepDone = 1;
        M2DutyCycle = 0;
      }
      else{
        if(M2ActualPoint < M2SetPoint){
          digitalWrite(M2_DIR, 1);
          //sbi (PORTB, 4);
          if(M2DutyCycle < MIN_DUTYCYCLE_M2_UP){
            M2DutyCycle = MIN_DUTYCYCLE_M2_UP;
          }
          analogWrite(M2_PWM, 255 - M2DutyCycle);
        }
        if(M2ActualPoint > M2SetPoint){
          digitalWrite(M2_DIR, 0);
          //cbi (PORTB, 4);
          if(M2DutyCycle < MIN_DUTYCYCLE_M2_DOWN){
            M2DutyCycle = MIN_DUTYCYCLE_M2_DOWN;
          }
          analogWrite(M2_PWM, M2DutyCycle);
        }
      }



      M3SensorReadOld = M3SensorRead;

      M3SensorRead = digitalRead(M3_S1);
      if(digitalRead(M3_S2) == 1){
        M3SensorRead += 2;
      }

      //M3SensorRead = 0;
      //if((PINC & 0b0010000) == 16){
      //  M3SensorRead = 1;
      //}
      //if((PINC & 0b00100000) == 32){
      //  M3SensorRead += 2;
      //}

      if(M3SensorRead != M3SensorReadOld){
        if(M3SensorRead == 0 && M3SensorReadOld == 2){
          M3ActualPoint--;
        }
        if(M3SensorRead == 1 && M3SensorReadOld == 0){
          M3ActualPoint--;
        }
        if(M3SensorRead == 3 && M3SensorReadOld == 1){
          M3ActualPoint--;
        }
        if(M3SensorRead == 2 && M3SensorReadOld == 3){
          M3ActualPoint--;
        }

        if(M3SensorRead == 0 && M3SensorReadOld == 1){
          M3ActualPoint++;
        }
        if(M3SensorRead == 1 && M3SensorReadOld == 3){
          M3ActualPoint++;
        }
        if(M3SensorRead == 3 && M3SensorReadOld == 2){
          M3ActualPoint++;
        }
        if(M3SensorRead == 2 && M3SensorReadOld == 0){
          M3ActualPoint++;
        }
      }

      if(abs(M3SetPoint - M3ActualPoint) < STEP_MARGIN_M3){
        analogWrite(M3_PWM, 0);
        digitalWrite(M3_DIR, 0);
        //cbi (PORTD, 2);
        M3StepDone = 1;
        M3DutyCycle = 0;
      }
      else{
        if(M3ActualPoint < M3SetPoint){
          digitalWrite(M3_DIR, 1);
          //sbi (PORTD, 2);
          if(M3DutyCycle < MIN_DUTYCYCLE_M3_UP){
            M3DutyCycle = MIN_DUTYCYCLE_M3_UP;
          }
          analogWrite(M3_PWM, 255 - M3DutyCycle);
        }
        if(M3ActualPoint > M3SetPoint){
          digitalWrite(M3_DIR, 0);
          //cbi (PORTD, 2);
          if(M3DutyCycle < MIN_DUTYCYCLE_M3_DOWN){
            M3DutyCycle = MIN_DUTYCYCLE_M3_DOWN;
          }
          analogWrite(M3_PWM, M3DutyCycle);
        }
      }

      if(verbose == 1){
        //if(M1SensorRead != M1SensorReadOld || M2SensorRead != M2SensorReadOld || M3SensorRead != M3SensorReadOld){
          printValues();
        //}
      }

      if(readByte == 0){//Don't wait stepTime if not data received
        millisStart = 0L;
      }

    }while( M1StepDone == 0 || M2StepDone == 0 || M3StepDone == 0 || millis() - millisStart < stepTime);



  readByte = 0;
  if (Serial.available() > 0){//if we get a valid byte, read from serial:
    //get incoming byte:
    readByte = Serial.read();
    Serial.print('r');   // send a 'r' to initiate next data from computer
  }//if (Serial.available() > 0){



  if(readByte > 0){

    if(readByte == 'a'){
      M1SetPoint++;
      M1DutyCycle = MAX_DUTYCYCLE_M1_UP;
      M1StepDone = 0;
    }
    if(readByte == 'b'){
      M1SetPoint--;
      M1DutyCycle = MAX_DUTYCYCLE_M1_DOWN;
      M1StepDone = 0;
    }

    if(readByte == 'A'){
      M1SetPoint += PULSES_PER_TURN;
      M1DutyCycle = MAX_DUTYCYCLE_M1_UP;
      M1StepDone = 0;
    }
    if(readByte == 'B'){
      M1SetPoint -= PULSES_PER_TURN;
      M1DutyCycle = MAX_DUTYCYCLE_M1_DOWN;
      M1StepDone = 0;
    }


    if(readByte == 'c'){
      M2SetPoint++;
      M2DutyCycle = MAX_DUTYCYCLE_M2_UP;
      M2StepDone = 0;
    }
    if(readByte == 'd'){
      M2SetPoint--;
      M2DutyCycle = MAX_DUTYCYCLE_M2_DOWN;
      M2StepDone = 0;
    }

    if(readByte == 'C'){
      M2SetPoint += PULSES_PER_TURN;
      M2DutyCycle = MAX_DUTYCYCLE_M2_UP;
      M2StepDone = 0;
    }
    if(readByte == 'D'){
      M2SetPoint -= PULSES_PER_TURN;
      M2DutyCycle = MAX_DUTYCYCLE_M2_DOWN;
      M2StepDone = 0;
    }


    if(readByte == 'e'){
      M3SetPoint++;
      M3DutyCycle = MAX_DUTYCYCLE_M3_UP;
      M3StepDone = 0;
    }
    if(readByte == 'f'){
      M3SetPoint--;
      M3DutyCycle = MAX_DUTYCYCLE_M3_DOWN;
      M3StepDone = 0;
    }

    if(readByte == 'E'){
      M3SetPoint += PULSES_PER_TURN;
      M3DutyCycle = MAX_DUTYCYCLE_M3_UP;
      M3StepDone = 0;
    }
    if(readByte == 'F'){
      M3SetPoint -= PULSES_PER_TURN;
      M3DutyCycle = MAX_DUTYCYCLE_M3_DOWN;
      M3StepDone = 0;
    }


    if(readByte == 'g'){
      ServoVal_1 += 1;
      if(ServoVal_1 > SERVOMAX_1){
        ServoVal_1 = SERVOMAX_1;
      }
    }
    if(readByte == 'h'){
      ServoVal_1 -= 1;
      if(ServoVal_1 < SERVOMIN_1){
        ServoVal_1 = SERVOMIN_1;
      }
    }

    if(readByte == 'i'){
      ServoVal_2 += 1;
      if(ServoVal_2 > SERVOMAX_2){
        ServoVal_2 = SERVOMAX_2;
      }
    }
    if(readByte == 'j'){
      ServoVal_2 -= 1;
      if(ServoVal_2 < SERVOMIN_2){
        ServoVal_2 = SERVOMIN_2;
      }
    }

    if(readByte == 'k'){
      ServoVal_3 += 1;
      if(ServoVal_3 > SERVOMAX_3){
        ServoVal_3 = SERVOMAX_3;
      }
    }
    if(readByte == 'l'){
      ServoVal_3 -= 1;
      if(ServoVal_3 < SERVOMIN_3){
        ServoVal_3 = SERVOMIN_3;
      }
    }

    if(readByte == 'm'){
      ServoVal_4 += 1;
      if(ServoVal_4 > SERVOMAX_4){
        ServoVal_4 = SERVOMAX_4;
      }
    }
    if(readByte == 'n'){
      ServoVal_4 -= 1;
      if(ServoVal_4 < SERVOMIN_4){
        ServoVal_4 = SERVOMIN_4;
      }
    }

    if(readByte == 'o'){
      ServoVal_5 += 1;
      if(ServoVal_5 > SERVOMAX_5){
        ServoVal_5 = SERVOMAX_5;
      }
    }
    if(readByte == 'p'){
      ServoVal_5 -= 1;
      if(ServoVal_5 < SERVOMIN_5){
        ServoVal_5 = SERVOMIN_5;
      }
    }


    if(readByte == 'S'){
      if(stepTime < 1000){
        stepTime+=10;
      }
    }
    if(readByte == 's'){
      if(stepTime > 10){
        stepTime-=10;
      }
    }

    if(readByte == 'v'){
      verbose = 1;
    }
    if(readByte == 'V'){
      verbose = 0;
    }



  }//if(readByte > 0){

}


